@@ -1,27 +1,32 @@
-#!/usr/bin/env python
 """This controller handles the create branch dialog."""
 
 
 import os
-from PyQt4.QtGui import QDialog
+from PyQt4 import QtGui
 
+import cola
+from cola import gitcmds
 from cola import utils
 from cola import qtutils
-from cola.views import CreateBranchView
+from cola.views import createbranch
 from cola.qobserver import QObserver
 
-def create_new_branch(model,parent,revision=''):
-    model = model.clone()
-    view = CreateBranchView(parent)
+def create_new_branch(revision=''):
+    """Launches a dialog for creating a new branch"""
+    model = cola.model().clone()
+    parent = QtGui.QApplication.instance().activeWindow()
+    view = createbranch.CreateBranchView(parent)
     ctl = CreateBranchController(model, view)
     model.set_revision(revision)
     view.show()
 
 class CreateBranchController(QObserver):
-    def init(self, model, view):
+    """Provides control for the "create branch" dialog"""
+    def __init__(self, model, view):
+        QObserver.__init__(self, model, view)
         self._remoteclicked = False
         self.add_observables('revision', 'local_branch')
-        self.add_callbacks(branch_list   = self.item_changed,
+        self.add_callbacks(branch_list   = self.branch_item_changed,
                            create_button = self.create_branch,
                            local_radio   = self.display_model,
                            remote_radio  = self.display_model,
@@ -31,18 +36,16 @@ class CreateBranchController(QObserver):
     #+--------------------------------------------------------------------
     #+ Qt callbacks
     def create_branch(self):
-        """This callback is called when the "Create Branch"
-        button is called."""
+        """Creates a branch; called by the "Create Branch" button"""
 
-        revision = self.model.get_revision()
-        branch = self.model.get_local_branch()
-        existing_branches = self.model.get_local_branches()
+        revision = self.model.revision
+        branch = self.model.local_branch
+        existing_branches = self.model.local_branches
 
         if not branch or not revision:
-            qtutils.information(
-                self.tr('Missing Data'),
-                self.tr('Please provide both a branch'
-                       +' name and revision expression.'))
+            qtutils.information('Missing Data',
+                                'Please provide both a branch '
+                                'name and revision expression.')
             return
 
         check_branch = False
@@ -50,10 +53,10 @@ class CreateBranchController(QObserver):
             if self.view.no_update_radio.isChecked():
                 msg = self.tr("Branch '%s' already exists.")
                 msg = unicode(msg) % branch
-                qtutils.information( self.tr('warning'), msg )
+                qtutils.information('warning', msg)
                 return
             # Whether we should prompt the user for lost commits
-            commits = self.model.rev_list_range(revision, branch)
+            commits = gitcmds.rev_list_range(revision, branch)
             check_branch = bool(commits)
 
         if check_branch:
@@ -77,33 +80,37 @@ class CreateBranchController(QObserver):
                 unicode(self.tr("Reset '%s'?")) % branch
                 ])
 
-            result = qtutils.question(self.view, self.tr('warning'),
-                                      '\n'.join(lines))
+            result = qtutils.question(self.view, 'warning', '\n'.join(lines))
             if not result:
                 return
 
+        # TODO handle fetch
         track = self.view.remote_radio.isChecked()
         fetch = self.view.fetch_checkbox.isChecked()
         ffwd = self.view.ffwd_only_radio.isChecked()
         reset = self.view.reset_radio.isChecked()
         chkout = self.view.checkout_checkbox.isChecked()
 
-        output = self.model.create_branch(branch, revision, track=track)
-        qtutils.show_output(output)
-        if chkout:
-            qtutils.show_output(self.model.git.checkout(branch,
-                                                        with_stderr=True))
+        status, output = self.model.create_branch(branch, revision, track=track)
+        qtutils.log(status, output)
+        if status == 0 and chkout:
+            status, output = self.model.git.checkout(branch,
+                                                     with_status=True,
+                                                     with_stderr=True)
+            qtutils.log(status, output)
         self.view.accept()
 
-    def item_changed(self, *rest):
-        """This callback is called when the item selection changes
-        in the branch_list."""
+    def branch_item_changed(self, *rest):
+        """This callback is called when the branch selection changes"""
 
+        # When the branch selection changes then we should update
+        # the "Revision Expression" accordingly.
         qlist = self.view.branch_list
-        ( row, selected ) = qtutils.get_selected_row(qlist)
-        if not selected: return
+        (row, selected) = qtutils.selected_row(qlist)
+        if not selected:
+            return
 
-        sources = self.__get_branch_sources()
+        sources = self._branch_sources()
         rev = sources[row]
 
         # Update the model with the selection
@@ -121,17 +128,18 @@ class CreateBranchController(QObserver):
         self.model.set_local_branch(branch)
         self._remoteclicked = False
 
-    ######################################################################
-
     def display_model(self):
-        branches = self.__get_branch_sources()
+        """Sets the branch list to the available branches
+        """
+        branches = self._branch_sources()
         qtutils.set_items(self.view.branch_list, branches)
 
-    def __get_branch_sources(self):
-        """Get the list of items for populating the branch root list."""
+    def _branch_sources(self):
+        """Get the list of items for populating the branch root list.
+        """
         if self.view.local_radio.isChecked():
-            return self.model.get_local_branches()
+            return self.model.local_branches
         elif self.view.remote_radio.isChecked():
-            return self.model.get_remote_branches()
+            return self.model.remote_branches
         elif self.view.tag_radio.isChecked():
-            return self.model.get_tags()
+            return self.model.tags
